<?php

namespace Drupal\eca_helper\Plugin\Action;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\eca_form\Plugin\Action\FormFieldActionBase;

/**
 * Set access to a form field.
 *
 * @Action(
 *   id = "eca_helper_form_field_set_value",
 *   label = @Translation("ECA Helper: Form field set value"),
 *   description = @Translation("Set form field value."),
 *   type = "form"
 * )
 */
class FormFieldSetValue extends FormFieldActionBase {

  /**
   * Whether to use form field value filters or not.
   *
   * @var bool
   */
  protected bool $useFilters = FALSE;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'key' => '',
      'value' => '',
      'array' => FALSE,
      'method' => 'append',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form['key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Element key'),
      '#description' => $this->t('The element key to set value for the form element. Example #title, #attributes.class'),
      '#default_value' => $this->configuration['key'],
      '#weight' => -49,
      '#eca_token_replacement' => TRUE,
    ];

    $form['value'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Value'),
      '#description' => $this->t('The default value to prepopulate.'),
      '#default_value' => $this->configuration['value'],
      '#weight' => -48,
      '#eca_token_replacement' => TRUE,
    ];

    $form['array'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Set array value'),
      '#description' => $this->t('Set or append data to array value.'),
      '#required' => FALSE,
      '#return_value' => 1,
      '#weight' => -47,
      '#default_value' => $this->configuration['array'],
    ];

    $form['method'] = [
      '#type' => 'select',
      '#title' => $this->t('Method'),
      '#default_value' => $this->configuration['method'],
      '#description' => $this->t('The method to set value when use Set array value.'),
      '#weight' => -40,
      '#options' => [
        'set:clear' => $this->t('Set and clear previous value'),
        'set:empty' => $this->t('Set only when empty'),
        'append' => $this->t('Append to exist value'),
        'remove' => $this->t('Remove value instead of adding it'),
      ],
    ];
    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['key'] = $form_state->getValue('key');
    $this->configuration['value'] = $form_state->getValue('value');
    $this->configuration['array'] = $form_state->getValue('array');
    $this->configuration['method'] = $form_state->getValue('method');
    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function doExecute(): void {
    if ($element = &$this->getTargetElement()) {
      $element = &$this->jumpToFirstFieldChild($element);
      $key = $this->tokenService->getOrReplace($this->configuration['key']);
      $value = $this->tokenService->getOrReplace($this->configuration['value']);
      $array = $this->tokenService->getOrReplace($this->configuration['array']);
      $method = $this->tokenService->getOrReplace($this->configuration['method']);
      if ($key && is_string($key) && mb_strlen($key)) {
        if ($array) {
          $array_value = NestedArray::getValue($element, explode(".", $key)) ?? [];
          if (!is_array($array_value)) {
            $array_value = [$array_value];
          }
          if ($method === 'set:clear') {
            $array_value[] = $value;
          }
          if ($method === 'set:empty' && empty($array_value)) {
            $array_value = [$value];
          }
          if ($method === 'append') {
            $array_value[] = $value;
          }
          if ($method === 'remove') {
            $array_value = array_values(array_diff($array_value, is_array($value) ? $value : [$value]));
          }
          NestedArray::setValue($element, explode(".", $key), $array_value);
          return;

        }
        NestedArray::setValue($element, explode(".", $key), $value);
      }
    }
  }

}
